from django.test import TestCase

from pseudo_cms import utils


class ConvertToHTMLTests(TestCase):

    reST_text = "This is text\n"\
                "==============="
    plain_text = "This is text\nThat's Cool"
    html = "<h4>This is the title</h4>"

    def _strip_tag(self, val):
        # django 2.0 uses plain <br>. Prior uses <br />
        return val.replace("<br />", "<br>")

    def test_turns_rst_to_html(self):
        self.assertTrue(
            "<h4>This is text</h4>" in utils.reST_to_html(self.reST_text))

    def test_turns_text_to_html_line_breaks(self):
        self.assertEqual("This is text<br>That&#39;s Cool",
                         self._strip_tag(utils.text_to_html(self.plain_text)))

    def test_convert_to_html_uses_rst(self):
        result = utils.convert_to_html(self.reST_text, utils.reST)
        self.assertTrue("<h4>This is text</h4>" in result)

    def test_convert_to_html_uses_html(self):
        result = utils.convert_to_html(self.html, utils.HTML)
        self.assertTrue("<h4>This is the title</h4>" in result)

    def test_convert_to_html_uses_plain_text(self):
        result = utils.convert_to_html(self.plain_text, utils.PLAIN_TEXT)
        self.assertTrue(
            "This is text<br>That&#39;s Cool" in self._strip_tag(result))



from django import test
from django.core.exceptions import ValidationError

from pseudo_cms import validators

__all__ = ('ValidURLExistsTests', )


@test.override_settings(ROOT_URLCONF='pseudo_cms.tests.testing_urls')
class ValidURLExistsTests(test.TestCase):
    def test_returns_none_when_url_path_exists(self):
        path = "/tests/path-one/"
        self.assertEqual(None, validators.internal_path_exists(path))

    def test_raises_validation_error_when_path_doesnt_exist(self):
        path = "/tests/bad-path/"
        self.assertRaises(ValidationError, validators.internal_path_exists,
                          path)



from django import test
from django.core.exceptions import ValidationError
from django.db import IntegrityError

from pseudo_cms import models, utils

__all__ = ('ContentModelTests', )


def get_content_model(save=False, **kwargs):
    m = models.Content(**kwargs)
    if save:
        m.save()
    return m


class ContentModelTests(test.TestCase):
    urls = 'pseudo_cms.tests.testing_urls'

    def test_cant_save_content_for_path_that_doesnt_exist(self):
        content = get_content_model(url="/tests/bad/path/")

        with self.assertRaises(ValidationError) as ctx:
            content.full_clean()

        self.assertEqual([u"'/tests/bad/path/' is not a valid url."],
                         ctx.exception.message_dict['url'])

    def test_url_must_be_unique(self):
        url = "/some/url/"
        get_content_model(save=True, url=url)
        with self.assertRaises(IntegrityError):
            get_content_model(save=True, url=url)

    def test_gets_content_for_page(self):
        url = "/some/url/"
        get_content_model(save=True, url=url)

        content_model = models.Content.get_for_page(url)
        self.assertEqual(url, content_model.url)

    def test_get_content_for_page_returns_none_when_not_found(self):
        content_model = models.Content.get_for_page("/random/")
        self.assertEqual(None, content_model)

    def test_orders_by_url(self):
        one = get_content_model(url="ZZZ", save=True)
        two = get_content_model(url="AAA", save=True)
        three = get_content_model(url="MMM", save=True)
        content_models = models.Content.objects.all()
        self.assertEqual([two, three, one], list(content_models))

    def test_body_html_gets_saved_when_html_type_is_used(self):
        body = "<h1>This is my body</h1>"

        c = get_content_model(save=True, body=body, content_format=utils.HTML)
        c.save()
        self.assertHTMLEqual(body, c.body_html)

    def test_body_html_gets_saved_when_rst_type_is_used(self):
        body = "This is a title\n"\
               "==============="

        c = get_content_model(save=True, body=body, content_format=utils.reST)
        c.save()
        self.assertTrue("<h4>This is a title</h4>" in c.body_html)

    def test_body_html_gets_saved_when_plain_text_is_used(self):
        body = "This is a title\n"\
               "And some content"

        c = get_content_model(
            save=True, body=body, content_format=utils.PLAIN_TEXT)
        c.save()
        self.assertTrue("This is a title<br>And some content" in
                        c.body_html.replace("<br />", "<br>"))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):
    def forwards(self, orm):
        # Adding field 'Content.page_tagline'
        db.add_column(
            'pseudo_cms_content',
            'page_tagline',
            self.gf('django.db.models.fields.CharField')(
                default='', max_length=120, blank=True),
            keep_default=False)

        # Adding index on 'Content', fields ['url']
        db.create_index('pseudo_cms_content', ['url'])

    def backwards(self, orm):
        # Removing index on 'Content', fields ['url']
        db.delete_index('pseudo_cms_content', ['url'])

        # Deleting field 'Content.page_tagline'
        db.delete_column('pseudo_cms_content', 'page_tagline')

    models = {
        'pseudo_cms.content': {
            'Meta': {
                'ordering': "('url',)",
                'object_name': 'Content'
            },
            'body': ('django.db.models.fields.TextField', [], {}),
            'body_html': ('django.db.models.fields.TextField', [], {}),
            'content_format': ('django.db.models.fields.CharField', [], {
                'default': "u'text'",
                'max_length': '50'
            }),
            'id': ('django.db.models.fields.AutoField', [], {
                'primary_key': 'True'
            }),
            'image': ('image_helper.fields.SizedImageField', [], {
                'max_length': '100',
                'blank': 'True'
            }),
            'meta_description': ('django.db.models.fields.CharField', [], {
                'max_length': '200'
            }),
            'page_tagline': ('django.db.models.fields.CharField', [], {
                'max_length': '120',
                'blank': 'True'
            }),
            'page_title': ('django.db.models.fields.CharField', [], {
                'max_length': '60'
            }),
            'title': ('django.db.models.fields.CharField', [], {
                'max_length': '100'
            }),
            'url': ('django.db.models.fields.CharField', [], {
                'unique': 'True',
                'max_length': '200',
                'db_index': 'True'
            })
        }
    }

    complete_apps = ['pseudo_cms']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import pseudo_cms.validators
import image_helper.fields


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Content',
            fields=[
                ('id', models.AutoField(
                    verbose_name='ID',
                    serialize=False,
                    auto_created=True,
                    primary_key=True)),
                ('url', models.CharField(
                    help_text=b'URL endpoint of page content belongs to.',
                    unique=True,
                    max_length=200,
                    db_index=True,
                    validators=[pseudo_cms.validators.internal_path_exists])),
                ('title', models.CharField(
                    help_text=b'Title that shows on searches / browser tab.',
                    max_length=100)),
                ('meta_description', models.CharField(
                    help_text=b'Description that shows in search engine.',
                    max_length=200)),
                ('page_title', models.CharField(
                    help_text=b'Title that shows on page.', max_length=60)),
                ('page_tagline', models.CharField(
                    help_text=b'Tagline / Subtitle for page',
                    max_length=120,
                    blank=True)),
                ('content_format', models.CharField(
                    default='text',
                    max_length=50,
                    choices=[('html', 'Raw HTML'),
                             ('reST', 'reStructuredText'), ('text',
                                                            'Plain Text')])),
                ('body', models.TextField()),
                ('body_html', models.TextField()),
                ('image', image_helper.fields.SizedImageField(
                    upload_to=b'cms/content', blank=True)),
            ],
            options={
                'ordering': ('url', ),
            },
            bases=(models.Model, ),
        ),
    ]



from docutils.core import publish_parts

from django.conf import settings
from django.utils.encoding import smart_str
from django.template.defaultfilters import linebreaksbr

HTML = u"html"
PLAIN_TEXT = u"text"
reST = u"reST"
CONTENT_FORMAT_CHOICES = (
    (HTML, u'Raw HTML'),
    (reST, u'reStructuredText'),
    (PLAIN_TEXT, u"Plain Text"),
)

DOCUTILS_SETTINGS = getattr(settings, 'DOCUTILS_SETTINGS', {
    'doctitle_xform': False,
    'initial_header_level': 4,
    'id_prefix': 's-',
})


def reST_to_html(text):
    return publish_parts(
        source=smart_str(text),
        writer_name="html",
        settings_overrides=DOCUTILS_SETTINGS)['fragment']


def text_to_html(text):
    return linebreaksbr(text)


def convert_to_html(text, content_type):
    if content_type == PLAIN_TEXT:
        html = text_to_html(text)
    elif content_type == reST:
        html = reST_to_html(text)
    else:
        html = text
    return html
